<?xml version="1.0"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:asm1="http://www.oasis-open.org/committees/cam/content" 
	xmlns:w2e="http://thisSheet" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xs="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="xsd xs w2e">
	<!-- CAM kit utility program -->
	<!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
		Authors: Martin ME Roberts, March, 2008 -->
	<!--
		Purpose  : Parse XSD schema into CAM template XML structure components.
		Creates XML instance to be used as Structure in CAM template editor.
		Original : 03/21/2008
		Initials : mmer
		CAM ver    : 1.1 
		Description : Part of a set of tools to ease alignment of XSD based validations and documentation
		and CAM templates. Generates an XML instance in CAM structure format
		Updated  - Inits - Change
		03/21/08   mmer   Initial release
		03/22/08   drrw    Added simple type handling - enumeration value
		03/23/08   drrw    Correct handling of ref and base lookups by nameoutp without namespace
		03/23/08   drrw    Resolve enclosing type 
		03/24/08   drrw    Corrected attributes with @ref
		03/24/08   drrw    Added attributeGroup handling
		03/24/08   drrw    Prevented type outputting twice when duplicate type definitions
		03/24/08   drrw    Resolve xsd:token value referencing
		03/24/08   mmer  Added option for CAM output or not
		03/25/08   drrw    Updated namespace handling - resolve to use xsd namespaces
		03/25/08   drrw    Added new parameter for rules, inline and anno generation
		03/26/08   drrw    Add check for people who have redefined base types to prevent looping
		03/26/08   drrw    Prevent failure if enumerated type occurs more than once with same name
		03/27/08   drrw    Tweak the check on enumerated type names to process element sequences
		03/27/08   drrw    Clean up use of // referencing using match[1] technique
		03/28/08   drrw    Added include and logic for CAM rule generation
		03/31/08   drrw    Produce baseline structure generation
		04/01/08   drrw    AttributeGroup @ref handling and attributeGroup within attributeGroup
		04/01/08   drrw    Check simpleType matching for attributes
		04/02/08   drrw    Catch multiple values in attribute and chkType (for // patterns - take first one)
		04/02/08   drrw    Fix missing complexType lookup for non-ns names
		04/11/08   drrw    Reconcile code changes with xsd2cam logic
		04/14/08   drrw    Element with simpletype restriction content
		04/14/08   drrw    Output comments and choices as comments
		04/14/08   drrw    Recursion warning message 
		04/16/08   mmer  Handle attributes on complexType 
		04/16/08   drrw    Resolve sequence within sequence 
		04/16/08   drrw    Add simpleType for element
		04/17/08   drrw    Comment out non-essential debugging messages
		04/21/08   drrw    Refine substring handling for [1] instance only
		04/21/08   drrw    Ambiguous attribute match
		04/30/08   drrw    Match type references by namespace
		05/01/08   drrw    Support type referencing a la UBL style
		05/04/08   drrw    Add xsd:all and xsd:group support
		05/04/08   drrw    Fix attributeGroup handling
		05/04/08   drrw    xsd:list type handling
		05/09/08   drrw    align with xsd2cam logic on default ns
		05/09/08   drrw    simpleType restriction referencing via chkType
		05/09/08   drrw    add annotation outputting option
		05/12/08   drrw    handle sequence within choice
	-->
	
	<xsl:output method="xml" indent="yes" omit-xml-declaration="yes"/>
	<xsl:param name="elementName"></xsl:param>
	<xsl:param name="complexTypeName"></xsl:param>
	<xsl:param name="logLevel" as="xsd:integer">1</xsl:param>
	<xsl:param name="CAM" as="xsd:boolean">false</xsl:param>
	<xsl:param name="CAMmode">anno</xsl:param>

	<xsl:variable name="defaultNS" select="namespace-uri-for-prefix('',/xsd:schema)"/>
	<xsl:include href="cam-utils.xsl"/>

	<xsl:template match="/">
		<xsl:variable name="element-to-process">
			<xsl:choose>
				<xsl:when test="string-length($elementName) = 0">
					<xsl:value-of select="xsd:schema/xsd:element[1]/@name"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$elementName"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:value-of select="w2e:message('beginning processing',3)"/>
		<xsl:variable name="doc">

		<xsl:choose>
			<xsl:when test="string-length($elementName) &gt; 0">
				<xsl:apply-templates select="/xsd:schema/xsd:element[@name=$element-to-process]"/>
			</xsl:when>
			<xsl:when test="string-length($complexTypeName) &gt; 0">
				<xsl:apply-templates select="//xsd:complexType[@name=$complexTypeName][1]" mode="start"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="//xsd:element[@name=$element-to-process][1]"/>
			</xsl:otherwise>
		</xsl:choose>
		</xsl:variable>
		<xsl:variable name="nss">
			<xsl:call-template name="getNs">
				<xsl:with-param name="context" select="$doc"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="cleanExample">
			<xsl:apply-templates select="$doc" mode="ns">
				<xsl:with-param name="nss" select="$nss"/>
			</xsl:apply-templates>
		</xsl:variable>
		<xsl:variable name="results">
			<xsl:apply-templates select="$cleanExample/*" mode="structure"/>
		</xsl:variable>
		<xsl:for-each select="$results/*">
			<xsl:copy>
				<xsl:copy-of select=".//namespace::*"/>
				<xsl:copy-of select="@*|node()"/>
			</xsl:copy>
		</xsl:for-each>
	</xsl:template>

	<xsl:template match="xsd:element">
		<xsl:param name="ancestors"/>
		<xsl:param name="ref-name"/>
		<xsl:param name="ref-attributes"/>

		<!-- xsl:value-of select="w2e:message(concat('element=','[',$ancestors[1],']',@name|@ref,' - ',@type),4)"/ --><!-- recursion debugging use this -->
		<xsl:value-of select="w2e:message(concat('element=',@name|@ref,' - ',@type),4)"/>
		<xsl:choose>
			<xsl:when test="not(@ref) and @type">
				<xsl:call-template name="outputElementWithType">
					<xsl:with-param name="ancestors" select="$ancestors"/>
					<xsl:with-param name="ref-name" select="$ref-name"/>
					<xsl:with-param name="ref-attributes" select="$ref-attributes"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="@ref">
				<!-- xsl:value-of select="w2e:message('OutputElementWithRef',4)"/ -->
				<xsl:variable name="ref-chk" select="@ref"/>
				<xsl:variable name="ref-attrs" select="@*"/>
				<xsl:variable name="elementName" select="@name" />
				<xsl:choose>
					<xsl:when test="count(//xsd:element[@substitutionGroup=$ref-chk]) &gt; 0">
						<xsl:variable name="subCount" select="count(distinct-values(//xsd:element[@substitutionGroup=$ref-chk]/@name))"/>
						<!-- xsl:value-of select="w2e:message(concat('substitutionGroup=',$ref-chk,' items=',$subCount ),4)"/ -->
						<xsl:variable name="thisType" select="@type"/>
						<xsl:variable name="ns">
							<xsl:choose>
								<xsl:when test="contains(@type,'xsd:')">
									<xsl:call-template name="namespace-uri-of-qname">
										<xsl:with-param name="qname" select="@name"/>
									</xsl:call-template>
								</xsl:when>
								<xsl:otherwise>
									<xsl:call-template name="namespace-uri-of-qname">
										<xsl:with-param name="qname" select="@type"/>
									</xsl:call-template>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						<!-- xsl:value-of select="w2e:message(concat('substitutionNS=',$ns),4)"/ -->
						
						<xsl:choose>
							<xsl:when test="$subCount = 1">
								<xsl:variable name="thisRefName" select="//xsd:element[@substitutionGroup=$ref-chk][1]/@name[1]"/>
								<xsl:variable name="matched" select="//xsd:element[@substitutionGroup=$ref-chk]"/>
								<xsl:apply-templates select="$matched[1]">
									<xsl:with-param name="ancestors" select="$ancestors"/>
									<xsl:with-param name="ref-name" select="$thisRefName[1]"/>
									<xsl:with-param name="ref-attributes" select="$ref-attrs"/>
								</xsl:apply-templates>
							</xsl:when>
							<xsl:otherwise>
								<xsl:variable name="items" select="subsequence(//xsd:element[@substitutionGroup=$ref-chk]/@name,1,$subCount)"/>
								<!-- xsl:element name="xsd:choice" -->
								<xsl:value-of select="w2e:message(concat('xsd:choice(group)=',$ref-chk),4)"/>
								<xsl:comment> elements are a choice </xsl:comment>
								<!-- xsl:variable name="firstChoice" select="$items[1]"/ -->
									<xsl:for-each select="$items">
										<xsl:variable name="thisName" select="."/>
										<!-- xsl:value-of select="w2e:message(concat('substitutionName=',$thisName),4)"/ -->
										<xsl:variable name="matched" select="//xsd:element[@substitutionGroup=$ref-chk and @name=$thisName]"/>
										<xsl:apply-templates select="$matched[1]">
											<xsl:with-param name="ancestors" select="$ancestors"/>
											<xsl:with-param name="ref-name" select="$thisName"/>
											<xsl:with-param name="ref-attributes" select="$ref-attrs"/>
										</xsl:apply-templates>
									</xsl:for-each>
								<!-- /xsl:element -->
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise>
					<xsl:call-template name="outputElementWithRef">
						<xsl:with-param name="ancestors" select="$ancestors"/>
					</xsl:call-template>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="count(xsd:simpleType/xsd:restriction/xsd:enumeration) &gt; 0">
				<!-- xsl:value-of select="w2e:message('OutputElementSimpleType',4)"/ -->
				<xsl:variable name="value" select="xsd:simpleType/xsd:restriction/xsd:enumeration[1]/@value"/>
				<xsl:element name="{@name}">
					<!-- xsl:for-each select="@*">
						<xsl:variable name="attrName" select="concat('xsd:',name())"/>
						<xsl:attribute name="{$attrName}" select="."/>
					</xsl:for-each -->
					<xsl:call-template name="outputEnumeration">
						<xsl:with-param name="typeName" select="@name"/>
						<xsl:with-param name="ns" select="@targetNamespace"/>
						<xsl:with-param name="enumValues" select="xsd:simpleType/xsd:restriction/xsd:enumeration"/>
						<xsl:with-param name="mode" select="'-E'"/>
					</xsl:call-template>
					<xsl:value-of select="w2e:type-to-example(concat('&quot;',$value[1],'&quot;'),'type')"/>
					<!-- xsl:value-of select="w2e:message(concat('closing fullname=',@name),3)"/ -->
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="w2e:message('Otherwise no @type',4)"/>
				<xsl:call-template name="outputElementWithNoType">
					<xsl:with-param name="ancestors" select="$ancestors"/>
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>

	<xsl:template match="xsd:complexType" mode="start">
		<!-- xsl:value-of select="w2e:message(concat('ComplexType(start)=',@name),4)"/ -->
			<xsl:element name="{@name}">
				<xsl:apply-templates select="."/>
			</xsl:element>
	</xsl:template>

	<xsl:template match="xsd:complexType">
		<xsl:param name="ns"/>
		<xsl:param name="ancestors"/>
		<!-- xsl:value-of select="w2e:message(concat('ComplexType(any)=',@name,'[',position(),']'),4)"/ -->
		<xsl:variable name="thisName" select="@name"/>
		<xsl:apply-templates select="xsd:attribute"/>
		<xsl:apply-templates select="xsd:attributeGroup"/>
		<xsl:apply-templates select="xsd:anyAttribute"/>
		<xsl:choose>
		<xsl:when test="count(//xsd:complexType[@name=$thisName]) =1">
			<!-- xsl:value-of select="w2e:message('ApplyingUniqueComplexType',4)"/ -->
			<!-- xsl:apply-templates select="xsd:attribute"/ -->
			<xsl:apply-templates select="xsd:choice|xsd:sequence">
				<xsl:with-param name="ancestors" select="$ancestors"/>
			</xsl:apply-templates>
		</xsl:when>
		<xsl:when test="count(xsd:sequence) =1">
			<!-- xsl:value-of select="w2e:message('ComplexTypeSimpleSequence',4)"/ -->
			<!-- xsl:apply-templates select="xsd:attribute"/ -->
			<xsl:apply-templates select="xsd:choice|xsd:sequence">
				<xsl:with-param name="ancestors" select="$ancestors"/>
			</xsl:apply-templates>
		</xsl:when>
		<xsl:when test="ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace[contains(.,$ns)][1]">
			<!-- xsl:value-of select="w2e:message('ApplyingComplexType',4)"/ -->
			<!-- xsl:apply-templates select="xsd:attribute"/ -->
			<xsl:apply-templates select="xsd:choice|xsd:sequence">
				<xsl:with-param name="ancestors" select="$ancestors"/>
			</xsl:apply-templates>
		</xsl:when>
			<xsl:otherwise><xsl:value-of select="w2e:message(concat('namespace=',$ns,'[',ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace,']'),4)"/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="xsd:complexType" mode="enclosed">
		<xsl:param name="ns"/>
		<!-- xsl:value-of select="w2e:message(concat('ComplexType(enclosed)=',@name,'[',position(),']'),4)"/ -->
		<!-- xsl:if test="ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace[contains(.,$ns)][1]">
			<xsl:value-of select="w2e:message('ApplyingComplexType',4)"/>
		</xsl:if -->
		<xsl:call-template name="outputEnclosedType">
			<xsl:with-param name="typeName" select="@name"/>
			<xsl:with-param name="typeNS" select="$ns"/>
		</xsl:call-template>
	</xsl:template>

	<xsl:template match="xsd:complexType[xsd:simpleContent/xsd:extension]">
		<xsl:param name="ancestors"/>
		<xsl:variable name="loopChk" select="if (string-length(substring-after(xsd:simpleContent/xsd:extension/@base,':')) = 0) then xsd:simpleContent/xsd:extension/@base else substring-after(xsd:simpleContent/xsd:extension/@base,':')"/>
		<!-- xsl:value-of select="w2e:message(concat('xsd:complexType[simpleContent][',@name,'+',$loopChk,']'),4)"/ -->
		<xsl:choose>
		<!-- enhanced loop namespace aware check -->
		<xsl:when test="not($loopChk=@name)">
		<xsl:apply-templates select="xsd:simpleContent/xsd:extension" mode="simple">
			<xsl:with-param name="ancestors" select="$ancestors"/>
		</xsl:apply-templates>
		</xsl:when>
		<xsl:when test="$loopChk=@name and(not(@base=$loopChk))">
			<xsl:variable name="nsPath">
				<xsl:call-template name="namespace-uri-of-qname">
				<xsl:with-param name="qname" select="@base"/>
			</xsl:call-template>
			</xsl:variable>
			<xsl:variable name="matchedType" select="//xsd:import[@namespace=$nsPath]/xsd:schema/xsd:complexType[@name=$loopChk]/xs:simpleContent/xs:extension"/>
			<xsl:apply-templates select="$matchedType[1]/xsd:attribute"/>
			<xsl:apply-templates select="$matchedType[1]/xsd:attributeGroup"/>
			<xsl:value-of select="w2e:type-to-example($matchedType[1]/@base,'type')"/>
		</xsl:when>
		<xsl:otherwise><xsl:value-of select="w2e:type-to-example(@name,'type')"/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="xsd:complexType[xsd:simpleContent/xsd:restriction]">
		<xsl:param name="ancestors"/>
		<xsl:variable name="loopChk" select="if (string-length(substring-after(xsd:simpleContent/xsd:restriction/@base,':')) = 0) then xsd:simpleContent/xsd:restriction/@base else substring-after(xsd:simpleContent/xsd:restriction/@base,':')"/>
		<!-- xsl:value-of select="w2e:message(concat('xsd:complexType[simpleContent][',@name,'+',$loopChk,']'),4)"/ -->
		<xsl:choose>
			<!-- enhanced loop namespace aware check -->
			<xsl:when test="not($loopChk=@name)">
				<xsl:apply-templates select="xsd:simpleContent/xsd:restriction" mode="simple">
					<xsl:with-param name="ancestors" select="$ancestors"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:when test="$loopChk=@name and(not(@base=$loopChk))">
				<xsl:variable name="nsPath">
					<xsl:call-template name="namespace-uri-of-qname">
						<xsl:with-param name="qname" select="@base"/>
					</xsl:call-template>
				</xsl:variable>
				<xsl:variable name="matchedType" select="//xsd:import[@namespace=$nsPath]/xsd:schema/xsd:complexType[@name=$loopChk]/xs:simpleContent/xs:restriction"/>
				<xsl:apply-templates select="$matchedType[1]/xsd:attribute"/>
				<xsl:apply-templates select="$matchedType[1]/xsd:attributeGroup"/>
				<xsl:value-of select="w2e:type-to-example($matchedType[1]/@base,'type')"/>
			</xsl:when>
			<xsl:otherwise><xsl:value-of select="w2e:type-to-example(@name,'type')"/></xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="xsd:complexType[xsd:complexContent]">
		<xsl:param name="ancestors"/>
		<xsl:param name="ns"/>
		<xsl:value-of select="w2e:message(concat('xsd:complexType[xsd:complexContent]:',@name,';','[',$ns,']'),4)"/>
		<xsl:if test="ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace[contains(.,$ns)][1]">
			<!-- xsl:value-of select="w2e:message(concat('xsd:complexContent:',@name,';','[',$ns,']'),4)"/ -->
			<xsl:apply-templates select="xsd:complexContent/xsd:extension" mode="complex">
				<xsl:with-param name="ancestors" select="$ancestors"/>
			</xsl:apply-templates>
			<xsl:apply-templates select="xsd:simpleContent/xsd:extension" mode="simple">
				<xsl:with-param name="ancestors" select="$ancestors"/>
			</xsl:apply-templates>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="xsd:group">
		<xsl:variable name="refName" select="@ref|@name"/>
		<xsl:value-of select="w2e:message(concat('xsd:group: ',$refName),4)"/>
		<xsl:variable name="matched" select="//complexType[@name=$refName]"/>
		<xsl:apply-templates select="$matched[1]"></xsl:apply-templates>
	</xsl:template>
	
	<xsl:template match="xsd:simpleContent" mode="simple">
		<xsl:param name="ancestors"/>
		<xsl:value-of select="w2e:message(concat('xsd:simpleContent: ',xsd:extension/@base),4)"/>
		<xsl:apply-templates select="xsd:extension" mode="simple">
			<xsl:with-param name="ancestors" select="$ancestors"/>
		</xsl:apply-templates>
	</xsl:template>
	
	<xsl:template match="xsd:simpleContent">
		<xsl:param name="ancestors"/>
		<xsl:value-of select="w2e:message(concat('xsd:simpleContent (any): ',xsd:extension/@base),4)"/>
		<xsl:apply-templates select="xsd:extension" mode="simple">
			<xsl:with-param name="ancestors" select="$ancestors"/>
		</xsl:apply-templates>
	</xsl:template>

	<xsl:template match="xsd:sequence">
		<xsl:param name="ancestors"/>
		<!-- xsl:value-of select="w2e:message(concat('xsd:sequence: ',xsd:element[1]/@name,' ',xsd:element[1]/@type),4)"/ -->
		<xsl:apply-templates select="xsd:element|xsd:choice">
			<xsl:with-param name="ancestors" select="$ancestors"/>
		</xsl:apply-templates>
		<xsl:apply-templates select="xsd:sequence">
			<xsl:with-param name="ancestors" select="$ancestors"/>
		</xsl:apply-templates>
	</xsl:template>

	<xsl:template match="xsd:choice">
		<xsl:param name="ancestors"/>
		<xsl:value-of select="w2e:message('xsd:choice(element)',4)"/>
		<xsl:comment> elements are a choice </xsl:comment>
		<!-- xsl:variable name="firstElement" select="xsd:element[1]"/ -->
		<xsl:choose>
			<xsl:when test="xsd:sequence">
				<xsl:for-each select="xsd:sequence">
					<xsl:comment> = sequence = </xsl:comment>
						<xsl:apply-templates select="xsd:element">
							<xsl:with-param name="ancestors" select="$ancestors"/>
						</xsl:apply-templates>
				</xsl:for-each>
				<xsl:apply-templates select="xsd:element">
					<xsl:with-param name="ancestors" select="$ancestors"/>
				</xsl:apply-templates>
				<xsl:comment> end choice </xsl:comment>
			</xsl:when>
			<xsl:otherwise>
					<xsl:apply-templates select="xsd:element">
						<xsl:with-param name="ancestors" select="$ancestors"/>
					</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="xs:all">
		<xsl:param name="ancestors"/>
		<xsl:value-of select="w2e:message('xsd:all(element)',4)"/>
		<xsl:comment> elements are an all group </xsl:comment>
		<xsl:apply-templates select="xsd:element">
			<xsl:with-param name="ancestors" select="$ancestors"/>
		</xsl:apply-templates>
	</xsl:template>
	
	<xsl:template match="xsd:restriction" mode="simple">
		<xsl:param name="ancestors"/>
		<xsl:variable name="type" select="if (string-length(substring-after(@base,':')) = 0) then @base else substring-after(@base,':')"></xsl:variable>
		<xsl:apply-templates select="xsd:attribute"/>
		<xsl:apply-templates select="xsd:attributeGroup"/>
		<xsl:apply-templates select="xsd:anyAttribute"/>
		<xsl:value-of select="w2e:message(concat('xsd:restriction (simple): ',$type),4)"/>
		<xsl:variable name="matched" select="//xsd:complexType[@name=$type]/xsd:simpleContent/xsd:extension"/>
		<xsl:choose>
			<xsl:when test="string-length($matched[1]/@base) &gt; 0"><xsl:value-of select="w2e:type-to-example($matched[1]/@base,'base')"/></xsl:when>
			<xsl:when test="string-length($type) &gt; 0"><xsl:value-of select="w2e:type-to-example($type,'base')"/></xsl:when>
			<xsl:otherwise><xsl:value-of select="concat('base=',@base)"/></xsl:otherwise>
		</xsl:choose>	
	</xsl:template>

	<xsl:template match="xsd:extension" mode="simple">
		<xsl:param name="ancestors"/>
		<xsl:variable name="type" select="if (string-length(substring-after(@base,':')) = 0) then @base else substring-after(@base,':')"></xsl:variable>
		<xsl:apply-templates select="xsd:attribute"/>
		<xsl:apply-templates select="xsd:attributeGroup"/>
		<xsl:apply-templates select="xsd:anyAttribute"/>
		<xsl:value-of select="w2e:message(concat('xsd:extension (simple): ',$type),4)"/>
		<xsl:choose>
			<xsl:when test="count(//xsd:complexType[@name=$type]) &gt; 0  and(not(count(//xsd:simpleType[@name=$type]/xsd:restriction) &gt;0))">
				<xsl:variable name="matched" select="//xsd:complexType[@name=$type]"/>
				<xsl:apply-templates select="$matched[1]">
				</xsl:apply-templates>
			</xsl:when>
			<xsl:otherwise>
			 <xsl:variable name="baseType">
			  <xsl:choose>
				<xsl:when test="count(//xsd:simpleType[@name=$type]/xsd:restriction/xsd:enumeration) &gt; 0">
					<xsl:variable name="matched" select="//xsd:simpleType[@name=$type]/xsd:restriction/xsd:enumeration"/>
					<!-- this is needed for when people redefine base types; hence duplicates can occur -->
				   	<xsl:value-of select="concat('&quot;',$matched[1]/@value,'&quot;')"/>
							<!-- xsl:value-of select="w2e:message(concat('xsd:enumeration: ',$type),4)"/ -->
							<!-- cannot do enumerations here - parent node not always there
								xsl:call-template name="outputEnumeration">
								<xsl:with-param name="typeName" select="$type"/>
								<xsl:with-param name="ns" select="@targetNamespace"/>
								<xsl:with-param name="enumValues" select="$matched"/>
							</xsl:call-template-->
				</xsl:when>
				<xsl:when test="count(//xsd:simpleType[@name=$type]/xsd:restriction/@base) &gt; 0">
					<xsl:variable name="matched" select="//xsd:simpleType[@name=$type]/xsd:restriction/@base"/>
					<xsl:value-of select="$matched[1]"/>
					<xsl:value-of select="w2e:message(concat('xsd:restriction: ',$type),4)"/>
					 	<!-- xsl:apply-templates select="xsd:attribute"/ -->
						<!-- xsl:apply-templates select="xsd:attributeGroup"/ -->
					<!-- xsl:call-template name="outputRestrictions">
								<xsl:with-param name="typeName" select="$type"/>
								<xsl:with-param name="ns" select="@targetNamespace"/>
								<xsl:with-param name="enumValues" select="$matched[1]"/>
							</xsl:call-template-->
				</xsl:when>
			    	<xsl:otherwise><xsl:value-of select="@base"/></xsl:otherwise>
			  </xsl:choose>
			 </xsl:variable><xsl:value-of select="w2e:message(concat('baseType ',$baseType),4)"/>
			<xsl:value-of select="w2e:type-to-example($baseType,'base')"/></xsl:otherwise>
		</xsl:choose>

	</xsl:template>

	<xsl:template match="xsd:extension" mode="complex">
		<xsl:param name="ancestors"/>
		<xsl:value-of select="w2e:message(concat('extension (complex): ',@base),4)"/>
		<xsl:variable name="type" select="if (string-length(substring-after(@base,':')) = 0) then @base else substring-after(@base,':')"/>
		<xsl:variable name="ns">
			<xsl:call-template name="namespace-uri-of-qname">
				<xsl:with-param name="qname" select="@base"/>
			</xsl:call-template>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('ns=',$ns),4)"/ -->
		<xsl:variable name="namespace-for-name">
			<xsl:value-of select="ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace"/>
		</xsl:variable>
		<xsl:variable name="ns-to-use">
			<xsl:value-of select="if (string-length($ns) = 0) then $namespace-for-name else $ns"/>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('namespace to use=',$ns-to-use,':',$type),4)"/ -->
		<xsl:apply-templates select="xsd:attribute"/>
		<xsl:apply-templates select="xsd:attributeGroup"/>
		<xsl:apply-templates select="xsd:anyAttribute"/>
		<xsl:variable name="matched" select="//xsd:complexType[@name=$type]"></xsl:variable>
		<xsl:apply-templates select="$matched[1]">
			<xsl:with-param name="ancestors" select="$ancestors"/>
			<xsl:with-param name="ns" select="$ns-to-use"/>
		</xsl:apply-templates>
		<xsl:choose>
		<xsl:when test="xsd:sequence">
			<xsl:apply-templates select="xsd:sequence">
				<xsl:with-param name="ancestors" select="$ancestors"/>
			</xsl:apply-templates>
		</xsl:when>
		<xsl:when test="xsd:choice">
			<xsl:apply-templates select="xsd:choice">
				<xsl:with-param name="ancestors" select="$ancestors"/>
			</xsl:apply-templates>
		</xsl:when>
		<xsl:when test="xsd:all">
			<xsl:apply-templates select="xsd:all">
					<xsl:with-param name="ancestors" select="$ancestors"/>
			</xsl:apply-templates>
		</xsl:when>
		<xsl:when test="xsd:group">
			<xsl:apply-templates select="xsd:group">
					<xsl:with-param name="ancestors" select="$ancestors"/>
			</xsl:apply-templates>
		</xsl:when>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="xsd:enumeration">
		<xsl:value-of select="w2e:message(concat('xsd:enumeration: ',@value),4)"/>
	</xsl:template>

	<xsl:template match="xsd:attributeGroup[@name]">
		<!-- xsl:value-of select="w2e:message(concat('xsd:attributeGroup: ',@name),4)"/ -->
		<xsl:apply-templates select="xsd:attribute"/>
		<xsl:apply-templates select="xsd:attributeGroup"/>
	</xsl:template>

	<xsl:template match="xsd:attributeGroup[@ref]">
		<!-- xsl:value-of select="w2e:message(concat('xsd:attributeGroup(ref): ',@ref),4)"/ -->
		<xsl:choose>
			<xsl:when test="count(xsd:attribute|xsd:attributeGroup) &gt; 0">
				<xsl:apply-templates select="xsd:attribute"/>
				<xsl:apply-templates select="xsd:attributeGroup"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="ref" select="if (string-length(substring-after(@ref,':')) = 0) then @ref else substring-after(@ref,':')"/>
				<xsl:choose>
				<xsl:when test="not(@ref=$ref)">
					<xsl:variable name="matched" select="//xsd:attributeGroup[@name=$ref]"/>
					<xsl:apply-templates select="$matched[1]"/>
				</xsl:when>
				<xsl:when test="@ref=$ref">
					<xsl:variable name="matched" select="//xsd:attributeGroup[@name=$ref]"/>
					<xsl:choose>			
					<xsl:when test="count($matched[1]/xsd:attribute) &gt; 0">
						<xsl:for-each select="$matched[1]/xsd:attribute">
							<xsl:variable name="attrType" select="if(string-length(@type) &gt; 0) then @type else 'string'"/>
							<xsl:call-template name="outputAttribute">
								<xsl:with-param name="name" select="@name"/>
								<xsl:with-param name="ns" select="''"/>
								<xsl:with-param name="value" select="w2e:type-to-example($attrType, 'attr')"/>
							</xsl:call-template>
						</xsl:for-each>
						<xsl:if test="count($matched[1]/xsd:anyAttribute) &gt; 0">
							<xsl:call-template name="outputAttribute">
								<xsl:with-param name="name" select="'anyAttribute'"/>
								<xsl:with-param name="ns" select="''"/>
								<xsl:with-param name="value" select="w2e:type-to-example('##any', 'attr')"/>
							</xsl:call-template>
						</xsl:if>
					</xsl:when>
					<xsl:otherwise>
						<xsl:call-template name="outputAttribute">
							<xsl:with-param name="name" select="@ref"/>
							<xsl:with-param name="ns" select="''"/>
							<xsl:with-param name="value" select="w2e:type-to-example(@ref, 'attr')"/>
						</xsl:call-template>
					</xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise>
					<xsl:call-template name="outputAttribute">
						<xsl:with-param name="name" select="@ref"/>
						<xsl:with-param name="ns" select="''"/>
						<xsl:with-param name="value" select="w2e:type-to-example(@ref, 'attr')"/>
					</xsl:call-template>
				</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="xsd:anyAttribute">
		<xsl:value-of select="w2e:message(concat('xsd:anyAttribute: ',@namespace,'+',@processContents),4)"/>
		<xsl:comment> <xsl:value-of select="concat('xsd:anyAttribute: ',@namespace,'+',@processContents)"/> </xsl:comment>
	</xsl:template>
	
	<xsl:template match="xsd:any">
		<xsl:value-of select="w2e:message(concat('xsd:any: ',@namespace,'+',@processContents),4)"/>
		<xsl:comment> <xsl:value-of select="concat('xsd:any: ',@namespace,'+',@processContents)"/> </xsl:comment>
	</xsl:template>

	<xsl:template match="xsd:attribute[@name and(not(xsd:simpleType/xsd:restriction))]">
		<xsl:variable name="baseType"><xsl:call-template name="chkType"><xsl:with-param name="type" select="@type"/></xsl:call-template></xsl:variable>
		<xsl:variable name="baseTypeChk"  select="if(string-length($baseType) &gt; 0) then $baseType else 'string'"/>
		<!-- xsl:value-of select="w2e:message(concat('xsd:attribute with name:',@name,'+',$baseTypeChk),4)"/ -->
		<xsl:call-template name="outputAttribute">
			<xsl:with-param name="name" select="@name"/>
			<xsl:with-param name="ns" select="''"/>
			<xsl:with-param name="value" select="w2e:type-to-example($baseTypeChk, 'attr')"/>
		</xsl:call-template>
		<xsl:variable name="name" select="@name"/>
		<!-- xsl:for-each select="@*">
			<xsl:if test="not(name()='name')">
				<xsl:variable name="attrName" select="concat('xsd:attr-',$name,'-',name())"/>
				<xsl:attribute name="{$attrName}" select="."/>
			</xsl:if>
		</xsl:for-each -->
		<xsl:variable name="typeChk" select="@type"/>
		<!-- xsl:value-of select="w2e:message(concat('chk xsd:enumerations for simpleType:',@name,'+',$typeChk),4)"/ -->
		<xsl:if test="count(//xsd:simpleType[@name=$typeChk]/xsd:restriction/xsd:enumeration) &gt; 0">
			<!-- xsl:value-of select="w2e:message(concat('xsd:enumerations for simpleType:',@name,'+',$typeChk),4)"/ -->
			<xsl:variable name="matched" select="//xsd:simpleType[@name=$typeChk]"/>
			<xsl:call-template name="outputEnumeration">
				<xsl:with-param name="typeName" select="$name"/>
				<xsl:with-param name="ns" select="@targetNamespace"/>
				<xsl:with-param name="enumValues" select="$matched[1]/xsd:restriction/xsd:enumeration"/>
				<xsl:with-param name="mode"/>
			</xsl:call-template>
		</xsl:if>
		<!-- xsl:attribute name="{@name}"><xsl:value-of select="w2e:type-to-example(@type, 'attr')"/>	</xsl:attribute -->
	</xsl:template>

	<xsl:template match="xsd:attribute[xsd:simpleType/xsd:restriction and @name]">
		<!-- xsl:value-of select="w2e:message(concat('xsd:attribute with simpleType:',@name),4)"/ -->
		<xsl:call-template name="outputAttribute">
			<xsl:with-param name="name" select="@name"/>
			<xsl:with-param name="ns" select="''"/>
			<xsl:with-param name="value" select="w2e:type-to-example(xsd:simpleType/xsd:restriction/@base, 'attr')"/>
		</xsl:call-template>
		
		<xsl:variable name="name" select="@name"/>
		<xsl:variable name="attrName" select="concat('xsd:attr-',$name,'-type')"/>
		<!-- xsl:attribute name="{$attrName}" select="xsd:simpleType/xsd:restriction/@base"/ -->

		<xsl:if test="count(xsd:simpleType/xsd:restriction/xsd:enumeration) &gt; 0">
			<!-- xsl:value-of select="w2e:message(concat('xsd:enumerations for base attribute:',$name),4)"/ -->
			<xsl:call-template name="outputEnumeration">
				<xsl:with-param name="typeName" select="$name"/>
				<xsl:with-param name="ns" select="@targetNamespace"/>
				<xsl:with-param name="enumValues" select="xsd:simpleType/xsd:restriction/xsd:enumeration"/>
				<xsl:with-param name="mode"/>
			</xsl:call-template>
		</xsl:if>
		<!-- xsl:attribute name="{@name}"><xsl:value-of select="w2e:type-to-example(xsd:simpleType/xsd:restriction/@base, 'attr')"/>	</xsl:attribute -->
	</xsl:template>

	<xsl:template match="xsd:attribute[@ref]">
		<xsl:variable name="ref" select="if (string-length(substring-after(@ref,':')) = 0) then @ref else substring-after(@ref,':')"/>
		<xsl:variable name="ns">
			<xsl:call-template name="namespace-uri-of-qname">
				<xsl:with-param name="qname" select="@ref"/>
			</xsl:call-template>
		</xsl:variable>
		
		<xsl:variable name="attrType">
			<xsl:choose>
				<xsl:when test="count(//xsd:attribute[@name=@ref]/@type) &gt; 0">
					<xsl:variable name="matched" select="//xsd:attribute[@name=@ref]/@type"/>
					<!-- this is needed for when people use include with same attribute definitions; hence duplicates can occur -->
					<xsl:value-of select="$matched[1]"/>
					</xsl:when>
				<xsl:when test="count(//xsd:attribute[@name=$ref]/@type) &gt; 0">
					<xsl:variable name="matched" select="//xsd:attribute[@name=$ref]/@type"/>
					<!-- this is needed for when people use include with same attribute definitions; hence duplicates can occur -->
					<xsl:value-of select="$matched[1]"/>
					</xsl:when>
				<xsl:otherwise><xsl:value-of select="$ref"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="baseType"><xsl:call-template name="chkType"><xsl:with-param name="type" select="$attrType"/></xsl:call-template></xsl:variable>
		<xsl:call-template name="outputAttribute">
			<xsl:with-param name="name" select="$ref"/>
			<xsl:with-param name="ns" select="$ns"/>
			<xsl:with-param name="value" select="w2e:type-to-example($baseType, 'attr')"/>
		</xsl:call-template>
		
		<!-- xsl:attribute name="{$ref}" namespace="{$ns}"><xsl:value-of select="w2e:type-to-example($attrType, 'attr')"/>	</xsl:attribute-->
	</xsl:template>
	
	<xsl:template name="chkType">
		<xsl:param name="type"/>
		<!-- check if there is a simpletype defined -->
		<xsl:variable name="simpleType" select="//xsd:simpleType[@name=$type]"/>
			<xsl:choose>
				<xsl:when test="count($simpleType) &gt; 0">
					<xsl:choose>
						<xsl:when test="count($simpleType[1]/xsd:restriction/xsd:enumeration) &gt; 0"><xsl:value-of select="$simpleType[1]/xsd:restriction/xsd:enumeration[1]/@value"/></xsl:when>
						<xsl:otherwise><xsl:value-of select="$simpleType[1]/xsd:restriction/@base"/></xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise><xsl:value-of select="$type"/></xsl:otherwise>
			</xsl:choose>
	</xsl:template>
	
	<xsl:template name="outputAttribute">
		<xsl:param name="name"/>
		<xsl:param name="ns"/>
		<xsl:param name="value"/>
		<xsl:choose>
			<xsl:when test="string-length($ns) &gt; 0">
				<xsl:attribute name="{$name}" namespace="{$ns}">
					<xsl:value-of select="$value"/>
				</xsl:attribute>
			</xsl:when>
			<xsl:otherwise>
				<xsl:if test="string-length($name) &gt; 0">
					<xsl:attribute name="{$name}">
						<xsl:value-of select="$value"/>
					</xsl:attribute>
				</xsl:if>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="outputEnumeration">
		<xsl:param name="typeName"/>
		<xsl:param name="ns"/>
		<xsl:param name="enumValues"/>
		<xsl:param name="mode"/>
		<!-- xsl:value-of select="w2e:message(concat('xsd:enumerations: ',$typeName),4)"/ -->
		<!-- xsl:variable name="values"><xsl:for-each select="$enumValues"><xsl:if test="./preceding-sibling::*"> |</xsl:if>'<xsl:value-of select="./@value"/>'</xsl:for-each></xsl:variable>
		<xsl:value-of select="w2e:message(concat('values: ',$values),4)"/>
		<xsl:attribute name="{concat('xsd:attr-',$typeName,'-enumValues',$mode)}">
			<xsl:value-of select="$values"/>
		</xsl:attribute -->
	</xsl:template>

	<!-- output templates and functions-->
	<xsl:template name="outputElementWithType">
		<xsl:param name="ancestors"/>
		<xsl:param name="ref-name"/>
		<xsl:param name="ref-attributes"/>
		<xsl:variable name="chkType" select="@type"/>
		<xsl:variable name="type">
		 <xsl:choose>
		 	<xsl:when test="count(//xsd:simpleType[@name=$chkType]/xsd:restriction/@base) &gt; 0">
		 		<xsl:variable name="matched" select="//xsd:simpleType[@name=$chkType]/xsd:restriction/@base"/>
		 		<xsl:value-of select="$matched[1]"/>
		 	</xsl:when>
		 	<xsl:when test="count(//xsd:complexType[@name=$chkType]/xsd:simpleContent/xsd:extension/@base) &gt; 0">
		 		<xsl:variable name="matched" select="//xsd:complexType[@name=$chkType]/xsd:simpleContent/xsd:extension/@base"/>
		 		<xsl:variable name="chkMatch" select="if (string-length(substring-after($matched[1],':')) = 0) then $matched[1] else substring-after($matched[1],':')"/>
		 		<xsl:value-of select="w2e:message(concat('MatchedType: ',@name,' - ',$chkMatch),4)"/>
		 		<xsl:choose>
		 			<xsl:when test="count(//xsd:simpleType[@name=$chkMatch]/xsd:restriction/@base) &gt; 0">
		 				<xsl:variable name="getType" select="//xsd:simpleType[@name=$chkMatch]/xsd:restriction/@base"/>
		 				<xsl:value-of select="if (string-length(substring-after($getType[1],':')) = 0) then $getType[1] else substring-after($getType[1],':')"/></xsl:when>
		 			<xsl:otherwise><xsl:value-of select="$chkMatch" /></xsl:otherwise>
		 		</xsl:choose>
		 	</xsl:when>  
		 	<xsl:otherwise><xsl:value-of select="if (string-length(substring-after(@type,':')) = 0) then @type else substring-after(@type,':')" /></xsl:otherwise>
		 </xsl:choose>
		</xsl:variable>
		<xsl:value-of select="w2e:message(concat('outputElementWithType: ',@name,' - ',$type,' - ',$chkType),4)"/>
		<xsl:variable name="namespace-for-name">
			<xsl:choose>
				<xsl:when test="count(ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace) &gt; 0">
					<xsl:value-of select="ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace"/>
				</xsl:when>
				<xsl:otherwise>asm1</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('$namespace-for-name=',$namespace-for-name),3)"/ -->
		<xsl:variable name="ns">
			<xsl:call-template name="namespace-uri-of-qname">
				<xsl:with-param name="qname" select="@type"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ns-to-use">
			<xsl:value-of select="if (string-length($ns) = 0) then $namespace-for-name else $ns"/>
		</xsl:variable>
		<xsl:variable name="prefix1">
			<xsl:call-template name="prefix-for-namespace">
				<xsl:with-param name="namespace-uri" select="$namespace-for-name"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="prefix">
			<!-- xsl:value-of select="if (string-length($prefix1) = 1) then concat(generate-id(),':') else $prefix1"/ -->
			<xsl:value-of select="if (string-length($prefix1) = 1) then ' ' else $prefix1"/>
		</xsl:variable>


		<xsl:variable name="fullname" select="if (string-length($prefix) = 1) then @name else concat($prefix,@name)"/>
		<!-- xsl:value-of select="w2e:message(concat('fullname=',$fullname),3)"/ -->
		
		<xsl:element name="{$fullname}" namespace="{$namespace-for-name}">
			<xsl:if test="$prefix1 != $prefix and(string-length($prefix) &gt; 0)">
				<xsl:namespace name="{substring-before($prefix,':')}" select="$namespace-for-name"/>
			</xsl:if>
			<!-- reference element rules output -->
			<!-- xsl:if test="string-length($ref-name) &gt; 0">
			<xsl:for-each select="$ref-attributes">
				<xsl:value-of select="w2e:message(concat($ref-name,' reference attr=',name(),'-',.),3)"/>
				<xsl:if test="not(contains(name(),':'))">
					<xsl:variable name="attrName" select="concat('xsd:',name())"/>
					<xsl:attribute name="{$attrName}" select="."/>
				</xsl:if>
			</xsl:for-each>
			</xsl:if -->
			
			<!-- rules being output -->
			<!-- xsl:for-each select="@*">
				<xsl:variable name="attrName" select="concat('xsd:',name())"/>
				<xsl:attribute name="{$attrName}" select="."/>
				</xsl:for-each -->
			
			<!-- see if there is a base type being used -->
			<xsl:if test="count(//xsd:complexType[@name=$type]/xsd:simpleContent/xsd:extension/@base) &gt; 0">
				<xsl:variable name="baseTypes" select="//xsd:complexType[@name=$type]/xsd:simpleContent/xsd:extension/@base"/>
				<xsl:variable name="baseType" select="if(contains($baseTypes[1],':')) then substring-after($baseTypes[1],':') else $baseTypes[1]"/>
				<xsl:variable name="resolvedType">
				<xsl:choose>
					<xsl:when test="count(//xsd:complexType[@name=$baseType]/xsd:simpleContent/xsd:extension/@base) &gt; 0">
						<xsl:variable name="typeFound" select="//xsd:complexType[@name=$baseType]/xsd:simpleContent/xsd:extension/@base"/>
						<xsl:value-of select="$typeFound[1]"/>
					</xsl:when>
					<xsl:when test="count(//xsd:complexType[@name=$baseTypes[1]]/xsd:simpleContent/xsd:extension/@base) &gt; 0">
						<xsl:variable name="typeFound" select="//xsd:complexType[@name=$baseTypes[1]]/xsd:simpleContent/xsd:extension/@base"/>
						<xsl:value-of select="$typeFound[1]"/>
					</xsl:when>
					<xsl:otherwise><xsl:value-of select="$baseTypes[1]"/></xsl:otherwise>
				</xsl:choose>
				</xsl:variable>
				
				<!-- xsl:attribute name="{'xsd:base'}" select="$resolvedType"/ -->
				<!-- Now see if there is an enumeration too -->
				<xsl:variable name="chkResolvedType" select="if (contains($resolvedType[1],':')) then substring-after($resolvedType[1],':') else $resolvedType[1]"/>
				<xsl:if test="count(//xsd:simpleType[@name=$chkResolvedType]/xsd:restriction/xsd:enumeration) &gt; 0">
					<!-- xsl:value-of select="w2e:message(concat('outputElementEnumeration: ',@name,' - ',$resolvedType),4)"/ -->
					<xsl:variable name="matched" select="//xsd:simpleType[@name=$chkResolvedType]"/>
					<xsl:call-template name="outputEnumeration">
						<xsl:with-param name="typeName" select="@name"/>
						<xsl:with-param name="ns" select="@targetNamespace"/>
						<xsl:with-param name="enumValues" select="$matched[1]/xsd:restriction/xsd:enumeration"/>
						<xsl:with-param name="mode" select="'-E'"/>
					</xsl:call-template>
				</xsl:if>
			</xsl:if>

			<xsl:value-of select="w2e:message(concat('@type name: ',$type),4)"/>
			<!-- xsl:value-of select="w2e:message(concat('match ',concat('(',@name,'+',@type,')'),'ancestors:',$ancestors),4)"/ -->
			<xsl:choose>
				<xsl:when test="not(contains($ancestors,concat('(',@name,'+',@type,')')))">
					<xsl:variable name="name1" select="@name"/>
					<xsl:variable name="type1" select="@type"/>
					<xsl:value-of select="w2e:message(concat('not enclosed $ns-to-use=',$ns-to-use,';$type=',$type),4)"/>
					
					<xsl:variable name="matched" select="if (count(//xsd:complexType[@name=$type])) then //xsd:complexType[@name=$type] else if (count(//xsd:complexType[@name=$chkType])) then //xsd:complexType[@name=$chkType] else //xsd:simpleType[@name=$chkType]"/>

					<!-- find the matched item that matches the namespace of the type reference -->
					<xsl:variable name="chkNS" select="$matched[1]/ancestor::xsd:schema[1]/@targetNamespace[1]"/>
					
					<xsl:value-of select="w2e:message(concat('chkNS ',$chkNS),4)"/>
					
					<xsl:choose>		
						<xsl:when test="$chkNS=$ns-to-use">
							<xsl:value-of select="w2e:message(concat('first match NS ',$chkNS),4)"/>
							<xsl:apply-templates select="$matched[1]">
								<xsl:with-param name="ns" select="$ns-to-use"/>
								<xsl:with-param name="ancestors" select="concat($ancestors,'(',$name1,'+',$type1,')')"/>
							</xsl:apply-templates>
						</xsl:when>
						<xsl:otherwise>
						<xsl:variable name="prefixNS"><!-- check namespace prefix of base item -->
							<xsl:choose>
							<xsl:when test="count(//xsd:complexType[@name=$chkType]/xsd:simpleContent/xsd:extension/@base) &gt; 0">
								<xsl:variable name="matched" select="//xsd:complexType[@name=$chkType]/xsd:simpleContent/xsd:extension/@base"/>
								<xsl:variable name="this" select="$matched[1]/parent::xsd:schema/parent::xsd:import/@namespace[1]"/>
								<xsl:value-of select="$this[1]"/>
							</xsl:when>
							<xsl:when test="count(//xsd:complexType[@name=$chkType]/xsd:simpleContent/xsd:restriction/@base) &gt; 0">
								<xsl:variable name="matched" select="//xsd:complexType[@name=$chkType]/xsd:simpleContent/xsd:restriction/@base"/>
								<xsl:variable name="this" select="$matched[1]/parent::xsd:schema/parent::xsd:import/@namespace[1]"/>
								<xsl:value-of select="$this[1]"/>
							</xsl:when>
							<xsl:otherwise/>
							</xsl:choose>
						</xsl:variable>
						<xsl:value-of select="w2e:message(concat('prefixNS ',$prefixNS),4)"/>
						<xsl:variable name="matchedRef">
							<xsl:for-each select="$matched">
								<xsl:variable name="chkNS1" select="parent::xsd:schema/parent::xsd:schema[1]/@targetNamespace[1]"/>
								<!-- xsl:value-of select="w2e:message(concat('compareNS ',$chkNS1),4)"/ -->
								<xsl:choose>
									<xsl:when test="string-length($chkNS1)=0">0</xsl:when>
									<xsl:when test="$chkNS1=$prefixNS and(string-length($prefixNS) &gt; 0)">1</xsl:when>
									<xsl:when test="$chkNS1=$ns-to-use">1</xsl:when>
									<xsl:otherwise>0</xsl:otherwise>
								</xsl:choose>
							</xsl:for-each>
						</xsl:variable>
						<xsl:choose>
							<xsl:when test="contains($matchedRef,'1')">
								<xsl:variable name="offset" select="string-length(concat(substring-before($matchedRef,'1'),'1'))"/>
								<xsl:value-of select="w2e:message(concat('matchedRef NS ',$offset),4)"/>
								<xsl:apply-templates select="$matched[$offset]">
									<xsl:with-param name="ns" select="$ns-to-use"/>
									<xsl:with-param name="ancestors" select="concat($ancestors,'(',$name1,'+',$type1,')')"/>
								</xsl:apply-templates>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="w2e:message(concat('matched base ',@name,'+',$type),4)"/>
								<xsl:apply-templates select="$matched[1]">
									<xsl:with-param name="ns" select="$ns-to-use"/>
									<xsl:with-param name="ancestors" select="concat($ancestors,'(',$name1,'+',$type1,')')"/>
								</xsl:apply-templates>
							</xsl:otherwise>
						</xsl:choose>
						
						</xsl:otherwise>
					</xsl:choose>					
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="w2e:message(concat('enclosed recursive:',$type[1]),4)"/>
					<xsl:comment>recursive definition - type = <xsl:value-of select="$type[1]"/></xsl:comment>
					<!--<xsl:apply-templates select="//xsd:complexType[@name=$type]" mode="enclosed">
						<xsl:with-param name="ns" select="$ns-to-use"/>
					</xsl:apply-templates>-->
				</xsl:otherwise>
			</xsl:choose>
			<xsl:choose>
				<xsl:when test="//xsd:simpleType[@name=$type]/xsd:list"><xsl:value-of select="w2e:type-to-example(concat('&quot;',//xsd:simpleType[@name=$type]/xsd:list[1]/@itemType,' list','&quot;'),'value')"/></xsl:when>
				<xsl:when test="//xsd:simpleType[@name=$type]"><xsl:value-of select="w2e:type-to-example(concat('&quot;',//xsd:simpleType[@name=$type]/xsd:restriction/xsd:enumeration[1]/@value,'&quot;'),'value')"/></xsl:when>
				<xsl:when test="//xsd:simpleType[@name=$chkType]">
					<xsl:variable name="typeBase" select="if (count(//xsd:simpleType[@name=$chkType]/xsd:restriction/xsd:enumeration) &gt; 0) then //xsd:simpleType[@name=$chkType]/xsd:restriction/xsd:enumeration[1]/@value else //xsd:simpleType[@name=$chkType]/xsd:restriction[1]/@base"/>
					<xsl:value-of select="if (count(//xsd:simpleType[@name=$chkType]/xsd:restriction/xsd:enumeration) &gt; 0) then w2e:type-to-example(concat('&quot;',$typeBase[1],'&quot;'),'value') else if (string-length($typeBase[1]) &gt; 0) then w2e:type-to-example($typeBase[1],'type') else  w2e:type-to-example($type,'type')"/>
				</xsl:when>
				<xsl:when test="not(//xsd:complexType[@name=$type]) and(not(//xsd:complexType[@name=$chkType]))"><xsl:value-of select="w2e:type-to-example($type,'type')"/></xsl:when>
			 <xsl:otherwise/>
			</xsl:choose>
			<!-- xsl:value-of select="w2e:message(concat('closing fullname=',$fullname),3)"/ -->
		</xsl:element>
	</xsl:template>

	<xsl:template name="outputElementWithNoType">
		<xsl:param name="ancestors"/>
		<!-- xsl:value-of select="w2e:message(concat('outputElementNoType: ',@name),4)"/ -->
		<xsl:variable name="type" select="if (string-length(substring-after(@type,':')) = 0) then @type else substring-after(@type,':') "/>
		<xsl:variable name="namespace-for-name">
			<xsl:choose>
				<xsl:when test="count(ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace) &gt; 0">
					<xsl:value-of select="ancestor::xsd:schema[@targetNamespace][1]/@targetNamespace"/></xsl:when>
				<xsl:otherwise>asm1</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('$namespace-for-name=',$namespace-for-name),3)"/ -->
		<xsl:variable name="ns">
			<xsl:call-template name="namespace-uri-of-qname">
				<xsl:with-param name="qname" select="@type"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ns-to-use">
			<xsl:value-of select="if (string-length($ns) = 0) then $namespace-for-name else $ns"/>
		</xsl:variable>
		<xsl:variable name="prefix1">
			<xsl:call-template name="prefix-for-namespace">
				<xsl:with-param name="namespace-uri" select="$namespace-for-name"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="prefix">
			<!-- xsl:value-of select="if (string-length($prefix1) = 1) then concat(generate-id(),':') else $prefix1"/ -->
			<xsl:value-of select="if (string-length($prefix1) = 1) then '' else $prefix1"/>
		</xsl:variable>
		<xsl:variable name="fullname" select="if (string-length($prefix) = 0) then @name else concat($prefix,@name)"/>
		<!-- xsl:value-of select="w2e:message(concat('fullname=',$fullname),3)"/ -->
		<xsl:element name="{$fullname}" namespace="{$namespace-for-name}">
			
			<xsl:if test="$prefix1 != $prefix and(string-length($prefix) &gt; 0)">
				<xsl:namespace name="{substring-before($prefix,':')}" select="$namespace-for-name"/>
			</xsl:if>
			<!--<xsl:apply-templates select="//xsd:complexType[@name=$fullname]/xsd:simpleContent/xsd:extension/xsd:attribute|xsd:attributeGroup"/>-->
			<xsl:choose>
			<xsl:when test="xsd:complexType">
				<xsl:apply-templates select="xsd:complexType">
					<xsl:with-param name="ancestors" select="concat($ancestors,'(',@name,')')"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:when test="xsd:simpleType/xsd:restriction/xsd:enumeration"><xsl:value-of select="xsd:simpleType/xsd:restriction/xsd:enumeration[1]/@value"/></xsl:when>
			<xsl:otherwise><xsl:text>%anyContent%</xsl:text></xsl:otherwise>
			</xsl:choose>
			<!-- xsl:value-of select="w2e:message(concat('closing fullname=',$fullname),3)"/ -->
		</xsl:element>
	</xsl:template>
	
	<xsl:template name="outputElementWithRef">
		<xsl:param name="ancestors"/>
		<!-- xsl:value-of select="w2e:message(concat('outputElementWithRef: ',@name),4)"/ -->
		<xsl:variable name="ref">
			<xsl:choose>
				<xsl:when test="count(//xsd:element[@name=@ref]) &gt; 0">
					<xsl:value-of select="@ref"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="if (string-length(substring-after(@ref,':')) = 0) then @ref else substring-after(@ref,':')"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('ref=',$ref),3)"/ -->
		<xsl:variable name="ref-attributes" select="@*"/>
		<xsl:for-each select="//xsd:element[@name=$ref]">
			<xsl:choose>
				<xsl:when test="position()=1">
					<!-- get first exact match only (included schemas may have duplicated definitions) -->
					<xsl:apply-templates select=".">
						<xsl:with-param name="ancestors" select="concat($ancestors,'(',$ref,')')"/>
						<xsl:with-param name="ref-name" select="$ref"/>
						<xsl:with-param name="ref-attributes" select="$ref-attributes"/>
					</xsl:apply-templates>
				</xsl:when>
				<xsl:otherwise>
					<xsl:comment>duplicate definition ignored: <xsl:value-of select="$ref"/>(<xsl:value-of select="position()"/>)</xsl:comment>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template name="outputEnclosedType">
		<xsl:param name="typeName"/>
		<xsl:param name="typeNS"/>
		<!-- xsl:value-of select="w2e:message(concat('outputElementEnclosed: ',@name),4)"/ -->
		<xsl:variable name="type" select="if (string-length(substring-after($typeName[1],':')) = 0) then $typeName else substring-after($typeName[1],':') "/>
		<xsl:variable name="namespace-for-name">
			<xsl:value-of select="$typeNS"/>
		</xsl:variable>
		<!-- xsl:value-of select="w2e:message(concat('$namespace-for-name=',$namespace-for-name),3)"/ -->
		<xsl:variable name="ns">
			<xsl:call-template name="namespace-uri-of-qname">
				<xsl:with-param name="qname" select="$typeName"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="ns-to-use">
			<xsl:value-of select="if (string-length($ns) = 0) then $namespace-for-name else $ns"/>
		</xsl:variable>
		<xsl:variable name="prefix">
			<xsl:call-template name="prefix-for-namespace">
				<xsl:with-param name="namespace-uri" select="$namespace-for-name"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="fullname" select="if (string-length($prefix) = 0) then $typeName else concat($prefix,$typeName)"/>
		<!-- xsl:value-of select="w2e:message(concat('fullname=',$fullname),3)"/ -->
		
		<xsl:element name="{$fullname}" namespace="{$namespace-for-name}">
			<xsl:apply-templates select="//xsd:complexType[@name=$typeName]/xsd:complexContent/xsd:extension/xsd:attribute|xsd:attributeGroup"/>
			<!-- xsl:value-of select="w2e:message(concat('closing fullname=',$fullname),3)"/ -->
		</xsl:element>
	</xsl:template>

	<xsl:template match="text()"/>
	<xsl:template match="choices"><xsl:value-of select="w2e:message('choices',3)"/><xsl:comment> select choice here </xsl:comment></xsl:template>
	
</xsl:stylesheet><!-- Stylus Studio meta-information - (c) 2004-2007. Progress Software Corporation. All rights reserved.

<metaInformation>
	<scenarios>
		<scenario default="yes" name="Scenario1" userelativepaths="yes" externalpreview="no" url="..\resources\Discovery.full.xml" htmlbaseurl="" outputurl="" processortype="saxon8" useresolver="yes" profilemode="0" profiledepth="" profilelength=""
		          urlprofilexml="" commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" validateoutput="no" validator="internal"
		          customvalidator="">
			<parameterValue name="elementName" value="'DiscoveryResponse'"/>
			<advancedProp name="sInitialMode" value=""/>
			<advancedProp name="bXsltOneIsOkay" value="true"/>
			<advancedProp name="bSchemaAware" value="true"/>
			<advancedProp name="bXml11" value="false"/>
			<advancedProp name="iValidation" value="0"/>
			<advancedProp name="bExtensions" value="true"/>
			<advancedProp name="iWhitespace" value="0"/>
			<advancedProp name="sInitialTemplate" value=""/>
			<advancedProp name="bTinyTree" value="true"/>
			<advancedProp name="bWarnings" value="true"/>
			<advancedProp name="bUseDTD" value="false"/>
			<advancedProp name="iErrorHandling" value="fatal"/>
		</scenario>
	</scenarios>
	<MapperMetaTag>
		<MapperInfo srcSchemaPathIsRelative="yes" srcSchemaInterpretAsXML="no" destSchemaPath="" destSchemaRoot="" destSchemaPathIsRelative="yes" destSchemaInterpretAsXML="no"/>
		<MapperBlockPosition></MapperBlockPosition>
		<TemplateContext></TemplateContext>
		<MapperFilter side="source"></MapperFilter>
	</MapperMetaTag>
</metaInformation>
-->
